<?php

namespace Me_Qr\Controller\Api;

use Me_Qr\Exceptions\QrCode\QrCodeExceptionInterface;
use Me_Qr\Exceptions\QrCode\QrCodeRequestDataException;
use Me_Qr\Services\ErrorHandler\ErrorHandlerService;
use Me_Qr\Services\QrProvider\QrProvider;
use Throwable;
use WP_REST_Controller;
use WP_REST_Request;
use WP_REST_Response;

class QrCodeLinkController extends WP_REST_Controller
{
    private QrProvider $qrProvider;
    private ErrorHandlerService $errorHandlerService;

    public function __construct(
        QrProvider $qrProvider,
        ErrorHandlerService $errorHandlerService
    ) {
        $this->qrProvider = $qrProvider;
        $this->errorHandlerService = $errorHandlerService;
        $this->namespace = 'me-qr/api';
    }

    public function register_routes(): void
    {
        register_rest_route( $this->namespace, "/get/qr", [
            [
                'methods'  => 'POST',
                'permission_callback' => '__return_true',
                'callback' => [$this, 'getQr'],
                'args'     => array(
                    'postId' => array(
                        'type' => 'int',
                        'required' => true,
                        'message' => 'Required post id not specified',
                    ),
                    'format' => array(
                        'type' => 'string',
                        'required' => false,
                        'validate_callback' => static function($param) {
                            if ( !in_array($param, ME_QR_VALID_QR_FORMATS, true)) {
                                return false;
                            }

                            return true;
                        },
                        'message' => "Available qr code format values: 'png', 'svg'",
                    ),
                ),
            ],
        ] );

        register_rest_route( $this->namespace, "/get/all-qr", [
            [
                'methods'  => 'POST',
                'permission_callback' => '__return_true',
                'callback' => [$this, 'getAllQr'],
                'args'     => array(
                    'postId' => array(
                        'type' => 'int',
                        'required' => true,
                        'message' => 'Required post id not specified',
                    ),
                ),
            ],
        ] );
    }

    public function getQr( WP_REST_Request $request ): WP_REST_Response
    {
        try {
            $postId = $request->get_param('postId');
            $format = $request->get_param('format');
            $link = get_permalink($postId);
            if ( !$link ) {
                throw new QrCodeRequestDataException('Post with given id was not found or does not exist');
            }

            $qrProviderDTO = $this->qrProvider->getQr($postId, $link, $format);

            return new WP_REST_Response([
                'qr_code' => $qrProviderDTO->getQrCode(),
            ]);
        } catch (QrCodeExceptionInterface $e) {
            return new WP_REST_Response($e->getMessage(), 400);
        } catch (Throwable $e) {
            $this->errorHandlerService->handleException($e);
            return new WP_REST_Response('Internal Server Error', 500);
        }
    }

    public function getAllQr( WP_REST_Request $request ): WP_REST_Response
    {
        try {
            $postId = $request->get_param('postId');
            $link = get_permalink($postId);
            if ( !$link ) {
                throw new QrCodeRequestDataException('Post with given id was not found or does not exist');
            }

            $allQrFormatProviderDTO = $this->qrProvider->getAllQrFormat($postId, $link);

            return new WP_REST_Response([
                'qr_codes' => [
                    'qr_png' => $allQrFormatProviderDTO->getPngQr()->getQrCode(),
                    'qr_svg' => $allQrFormatProviderDTO->getSvgQr()->getQrCode(),
                ],
            ]);
        } catch (QrCodeExceptionInterface $e) {
            return new WP_REST_Response($e->getMessage(), 400);
        } catch (Throwable $e) {
            $this->errorHandlerService->handleException($e);
            return new WP_REST_Response('Internal Server Error', 500);
        }
    }
}
