<?php

namespace Me_Qr\Services\Validation\Assert\Types;

use JsonException;
use Me_Qr\Services\Validation\Exceptions\ValidationException;
use Me_Qr\Services\Validation\Exceptions\ValidationExceptionInterface;
use Me_Qr\Services\Validation\Exceptions\ValidationSystemError;
use ReflectionProperty;

abstract class AbstractValidationType implements ValidationTypeInterface
{
    /** This is the title of the invalid value message attribute. */
    public const ERROR_MESSAGE_PROPERTY = 'errorMessage';
    protected const PROPERTY_KEY = '{{ property }}';
    protected const VALUE_KEY = '{{ value }}';
    protected const VALUE_TYPE_KEY = '{{ type }}';

    /** This is the name of the currently validated property */
    protected $propertyName;

    /** This is the value of the class property that is currently being validated */
    protected $value;

    /** This is an error message if the value is not valid */
    protected ?string $errorMessage = 'The specified value "{{ value }}" of the property {{ property }}, is not valid.';

    /**
     * Returns an array of settings for the current validation class property
     *
     * @throws ValidationSystemError
     */
    protected function parsePropertyArray(ReflectionProperty $property): array
    {
        $annotation = $property->getDocComment();
        $annotation = str_replace(['*', PHP_EOL], '', $annotation);

        $pattern = "/(?<=" . static::MODULE_TITLE . "[(])" . "[^)]*" . "(?=[)])/";
        preg_match_all($pattern, $annotation, $matches);

        $jsonProperty = $matches[0][0] ?? null;
        if ( !$jsonProperty ) {
            return [];
        }
        $jsonProperty = str_replace([',}', ', }'], '}', $jsonProperty);

        try {
            return json_decode($jsonProperty, true, 512, JSON_THROW_ON_ERROR);
        } catch (JsonException $e) {
            throw $this->createSystemException();
        }
    }

    /**
     * @throws ValidationExceptionInterface
     */
    public function validate(ReflectionProperty $property, object $object): ?ValidationExceptionInterface
    {
        $propertyParams = $this->parsePropertyArray($property);
        $this->declareDefault($propertyParams, $property, $object);
        $this->declare($propertyParams);

        if ( !$this->isValid() ) {
            return $this->createValidationException();
        }

        return null;
    }

    /**
     * @throws ValidationExceptionInterface
     */
    protected function declareDefault(array $propertyParams, ReflectionProperty $property, object $object): void
    {
        $property->setAccessible(true);
        $this->propertyName = $property->getName();
        $this->value = $property->getValue($object);

        $errorMessage = $propertyParams[self::ERROR_MESSAGE_PROPERTY] ?? null;
        if ( $errorMessage && is_string($errorMessage) ) {
            $this->errorMessage = $errorMessage;
        }
    }

    /**
     * @throws ValidationExceptionInterface
     */
    public function declare(array $propertyParams): void
    {
        // Can be overridden to define specific validation type properties...
    }

    protected function createSystemException(): ValidationSystemError
    {
        return new ValidationSystemError();
    }

    protected function createValidationException(): ValidationException
    {
        $message = str_replace(
            [self::PROPERTY_KEY, self::VALUE_KEY, self::VALUE_TYPE_KEY],
            [$this->propertyName, $this->value, gettype($this->value)],
            $this->errorMessage
        );

        $message = $this->changeErrorMessage($message);

        return new ValidationException($message);
    }

    protected function changeErrorMessage(string $message): string
    {
        // Here you can change the final validation error message.

        return $message;
    }
}
